#!/usr/bin/env python3
# -*- Mode: python; tab-width: 4; indent-tabs-mode:nil; coding:utf-8 -*-
#
# Copyright (c) 2021 Authors and contributors
#
# Released under the GNU Public Licence, v2 or any higher version
# SPDX-License-Identifier: GPL-2.0-or-later
"""Command line interfaces for MDAnalysis analysis classes."""

from mdacli.cli import cli

__all__ = ["cli"]

__version__ = '0.1.12'
