/* custom MDAnalysis styling (User Guide theme) */

/* See https://github.com/MDAnalysis/mdanalysis/wiki/MDAnalysis-theme-colours */
/* MDAnalysis orange: #FF9200 */
/* MDAnalysis gray: #808080 */
/* MDAnalysis white: #FFFFFF */
/* MDAnalysis black: #000000 */
/* Darker orange: e76900 */
/* Even darker orange: #a24900 */
/* RTD dark grey: #343131 */
/* RTD light grey: #e6e6e6 */

/* -- page layout --------------------------------------------------------- */

body {
    font-family: 'PT Sans', Helvetica, Arial, 'sans-serif';
    font-size: 17px;    
}

div.body {
    color: #000000;
}

div.sphinxsidebar a:hover {
    text-decoration: none !important;
}

div.sphinxsidebar p {
    color: #808080;
}

/* Home MDAnalysis colour */
.wy-side-nav-search > a {
    color: #343131;
}

/* Side MDAnalysis version colour */
.wy-side-nav-search > div.version {
    color: #808080;
}

/* Menubar caption colour */
div.wy-menu-vertical span.caption-text {
    color: #FF9200;
}

/* Mobile layout menubar option */
nav.wy-nav-top {
    background: #343131;
}

/* Menu search bar outline (default blue) */
.wy-side-nav-search input[type="text"] {
    border-color: #808080;
}


/* -- body styles --------------------------------------------------------- */

/* Different coloured links for sidebar vs body) */
div.rst-content a {
    color: #FF9200;
    text-decoration: none;
}

div.rst-content a:visited {
    color: #FF9200;
}
 
a:hover {
    color: #FF9200 !important;
    text-decoration: underline;
}


pre, tt, code {
    font-family: Menlo, Monaco, 'Courier New', monospace
}


div.body h1 {
    font-weight: bolder;
}

a.headerlink {
    color: #808080;
    font-size: 0.8em;
    padding: 0 4px 0 4px;
    text-decoration: none;
}
 
a.headerlink:hover {
    background-color: #808080;
    color: #fff;
}

/* ------- admonition boxes ------- */

div.admonition {
    margin: 10px 0px;
    padding: 10px 10px;
}

div.admonition p.admonition-title {
    font-size: 100%;
    font-weight: bolder;
}

/* ----- Tables ----- */

/* override table width restrictions */
/* wrap tables instead of scrolling */
@media screen and (min-width: 767px) {

    .wy-table-responsive table td, .wy-table-responsive table th {
       /* !important prevents the common CSS stylesheets from overriding
          this as on RTD they are loaded after this stylesheet */
       white-space: normal !important;
    }
 
    .wy-table-responsive {
       overflow: visible !important;
       max-width: 100% !important;
    }
 }

/* ----- Field lists ------ */

.section > dl.field-list {
    display: flex;
    flex-wrap: wrap;
    margin: 0;
    padding: 0;
}

dl.field-list > dt::after {
    content: ":";
}

.rst-content dl:not(.docutils) dt {
    background: none;
    color: #000000;
    border-top: none;
}

.section > dl.field-list dt {
    margin: 0;
    padding: 0;
    flex-basis: 20%;
    display: block;
}

.section > dl.field-list > dd {
    flex-basis: 70%;
    margin: 0;
}

.section > dl.field-list > dd p {
    margin: 0;
}

/* ----- MDAnalysis coloured elements ------ */

.rst-content dl.class dt, .rst-content dl.function dt {
    color: #ca6500;
    background: #FFEBD0;
    border-top: solid 3px #FF9200;
}

.rst-content .viewcode-link, .rst-content .viewcode-back {
    color: #808080;
}

.rst-content .guilabel {
    background: #efefef;
    border: 1px solid #808080;
}


.rst-content .seealso p.admonition-title {
    background: #808080;
}

.rst-content .seealso {
    background: #e3e3e3;
}

.rst-content  .error p.admonition-title, .rst-content  .warning p.admonition-title {
    background: #F45F4B;
}

.rst-content .error, .rst-content .warning {
    background: #FFEEED;
}

.rst-content .caution p.admonition-title, .rst-content .note p.admonition-title, .rst-content .important p.admonition-title  {
    background: #FF9200;
}

.rst-content .caution, .rst-content .note, .rst-content .important {
    background: #FFEBD0;
}

.rst-content code:not(.xref).literal {
    color: #ca6500;
}
